// This example shows how to read 4 items at once, and display their values, timestamps and qualities.
// It is designed to only use C++ language features and libraries that are portable across compilers.

// Uses: STLSoft C and C++ Libraries, http://sourceforge.net/projects/stlsoft
// Uses: VOLE - A Neat C++ COM/Automation Driver, http://sourceforge.net/projects/vole
// You need to properly set the include directories to STLSoft and VOLE libraries.

// Tested with libraries: STLSoft 1.9.124, VOLE 0.7.3
// Tested with compiler: Microsoft (R) C/C++ Optimizing Compiler Version 17.00.50727.1 for x86 (Visual Studio 2012)
// Tested with compiler: Microsoft (R) C/C++ Optimizing Compiler Version 17.00.50727.1 for x64 (Visual Studio 2012)

#include <vole/vole.hpp>
#include <comstl/util/initialisers.hpp>
#include <stdlib.h>

int main()
{
	using comstl::variant;
	using vole::object;
	using vole::collection;

	// Initialize OLE libraries
	comstl::ole_initialiser oleinit;

	// Instatiate the EasyOPC-DA client object
	object client = object::create("OpcLabs.EasyOpc.DataAccess.EasyDAClient", CLSCTX_ALL, vole::coercion_level::valueCoercion);

	//
	object readItemArguments1 = object::create("OpcLabs.EasyOpc.DataAccess.OperationModel.DAReadItemArguments", CLSCTX_ALL, vole::coercion_level::valueCoercion);
	readItemArguments1.get_property<object>(L"ServerDescriptor").put_property<comstl::variant>(L"ServerClass", comstl::variant(L"OPCLabs.KitServer.2"));
	readItemArguments1.get_property<object>(L"ItemDescriptor").put_property<comstl::variant>(L"ItemID", comstl::variant(L"Simulation.Random"));

	object readItemArguments2 = object::create("OpcLabs.EasyOpc.DataAccess.OperationModel.DAReadItemArguments", CLSCTX_ALL, vole::coercion_level::valueCoercion);
	readItemArguments2.get_property<object>(L"ServerDescriptor").put_property<comstl::variant>(L"ServerClass", comstl::variant(L"OPCLabs.KitServer.2"));
	readItemArguments2.get_property<object>(L"ItemDescriptor").put_property<comstl::variant>(L"ItemID", comstl::variant(L"Trends.Ramp (1 min)"));

	object readItemArguments3 = object::create("OpcLabs.EasyOpc.DataAccess.OperationModel.DAReadItemArguments", CLSCTX_ALL, vole::coercion_level::valueCoercion);
	readItemArguments3.get_property<object>(L"ServerDescriptor").put_property<comstl::variant>(L"ServerClass", comstl::variant(L"OPCLabs.KitServer.2"));
	readItemArguments3.get_property<object>(L"ItemDescriptor").put_property<comstl::variant>(L"ItemID", comstl::variant(L"Trends.Sine (1 min)"));

	object readItemArguments4 = object::create("OpcLabs.EasyOpc.DataAccess.OperationModel.DAReadItemArguments", CLSCTX_ALL, vole::coercion_level::valueCoercion);
	readItemArguments4.get_property<object>(L"ServerDescriptor").put_property<comstl::variant>(L"ServerClass", comstl::variant(L"OPCLabs.KitServer.2"));
	readItemArguments4.get_property<object>(L"ItemDescriptor").put_property<comstl::variant>(L"ItemID", comstl::variant(L"Simulation.Register_I4"));

	// Create safearray of variants
	variant argumentArray;
	argumentArray.vt = VT_ARRAY | VT_VARIANT;
	SAFEARRAYBOUND sab;
	sab.lLbound = 0; sab.cElements = 4;
	argumentArray.parray = SafeArrayCreate(VT_VARIANT, 1, &sab);

	// Fill safearray with read item arguments
	LONG index;
	
	index = 0; variant element1(readItemArguments1.get(), true); SafeArrayPutElement(argumentArray.parray, &index, &element1);
	index = 1; variant element2(readItemArguments2.get(), true); SafeArrayPutElement(argumentArray.parray, &index, &element2);
	index = 2; variant element3(readItemArguments3.get(), true); SafeArrayPutElement(argumentArray.parray, &index, &element3);
	index = 3; variant element4(readItemArguments4.get(), true); SafeArrayPutElement(argumentArray.parray, &index, &element4);

	variant results = client.invoke_method<variant>(L"ReadMultipleItems", argumentArray);

	SAFEARRAY* resultArray = results.parray;
	for (LONG i = 0; i <= 3; i++)
	{
		// Each element of the result array is a DAVtqResult object; obtain it
		variant element;
		SafeArrayGetElement(resultArray, &i, &element);
		object daVtqResult(element);

		// Obtain a DAVtq object (value/timestamp/quality) from the result 
		object daVtq = daVtqResult.get_property<object>(L"Vtq");

		// Now we can obtain the individual properties, or convert it to string
		variant string = daVtq.invoke_method<variant>(L"ToString");
		wprintf(L"results(%d).Vtq.ToString(): %s\n", (int)i, string.bstrVal);
	}

	char line[80];
	puts("Press Enter to continue...");
	fgets(line, sizeof(line), stdin);

	return EXIT_SUCCESS;
}
